using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

using BigSock.Service;
using BigSock.UI;
using BigSock.Item;
using BigSock.Interact;

using UnityEditor;

namespace BigSock.UI {
    public class ChestDisplay : MonoBehaviour
    {
        public Button closeButton;
        public List<IItem> items;
        public List<Button> buttons;
        public GameObject other;
        public GameObject chest;

        PlayerController player;

        void Start() {
            var itemButtonLocation = transform.Find("ItemBox").Find("ItemBoxBackground");
            buttons = new List<Button>();

            // Gets button gameobjects and adds onClick listener
            for (int i = 0; i < 3; i++) {
                var button = itemButtonLocation.transform.Find("ButtonItem"+(i+1).ToString()).GetComponent<Button>();
                int j = i;
                button.onClick.AddListener(delegate {ItemPicked(j); });
                buttons.Add(button);
            }

            // The X button :)
            closeButton = transform.Find("CloseButton").GetComponent<Button>();
            closeButton.onClick.AddListener(manuallyCloseWindow);

            // list of random items
            items = chest.transform.GetComponent<Chest>().GetChestItems();

            // Sets Item icon and name
            for (int i = 0; i < buttons.Count; i++) {
                buttons[i].gameObject.transform.Find("Item").GetComponent<Image>().sprite = ItemService.SINGLETON.Get(items[i].Id).Icon;
                buttons[i].gameObject.transform.Find("ItemName").GetComponent<TMPro.TMP_Text>().text = ItemService.SINGLETON.Get(items[i].Id).Name;
            }

            this.other = GameObject.FindWithTag("Player");
            player = other.GetComponent<PlayerController>();

            if (player.Inventory.AddItem(null) == -1) {
            for (int i = 0; i < buttons.Count; i++) {
                buttons[i].interactable = false;
            }
                transform.Find("Banner").Find("BannerBackground").Find("BannerExtraText").GetComponent<TMPro.TMP_Text>().text = "Inventory is full, free up some space to select an item";
            }
        }

        public void ShowItemDescription(GameObject button) {
            string[] buttonNumber = button.name.Split("ButtonItem"); 
            button.transform.Find("ItemDescription").GetComponent<TMPro.TMP_Text>().text = items[int.Parse(buttonNumber[1])-1].GetToolTip();
            button.transform.Find("Item").GetComponent<Image>().color = Color.clear;
        }

        public void HideItemDescription(GameObject button) {
            button.transform.Find("ItemDescription").GetComponent<TMPro.TMP_Text>().text = "";
            button.transform.Find("Item").GetComponent<Image>().color = Color.white;
        }

        private void Update() {
            if (Input.GetKeyDown(KeyCode.Escape)){ //|| Input.GetKeyDown(KeyCode.Mouse1)) {
					manuallyCloseWindow();
				}
        }

        private void manuallyCloseWindow() {
            chest.transform.GetComponent<Chest>().ItemNotPicked();
            Destroy(this.gameObject);
        }

        // Gets the chest element the display belongs to
        public void setChestObject(GameObject chestInstance) {
            chest = chestInstance;
        }

        void ItemPicked(int number) {
            player.TryPickUpItem(ItemService.SINGLETON.Get(items[number].Id));
            
            Destroy(this.gameObject);
        }
    }
}
